import copy

# データの実体は、メモリ上のオブジェクト管理区域に保管されていて、変数にはデータ型の情報とその保管場所のアドレス情報しか書かれていない。そのため、変数list_1も変数list_2もデータの実体は１つで、同じ実体にアクセスしている。この関係は、関数の実引数（呼び出し元の引数）と仮引数（呼び出し先の引数）にも当てはまる。
list_1 = [0, 1, 2, 3]
# 代入演算子を使ったリストのコピー（名札だけが複製される）
list_2 = list_1
print(list_2)  # [0, 1, 2, 3]
list_1[0] = -1
print(list_1)  # [-1, 1, 2, 3]
print(list_2)  # [-1, 1, 2, 3]
list_2[3] = 10
print(list_1)  # [-1, 1, 2, 10]

# copyモジュールのcopy関数を使ったリストのコピーでは、名札だけでなく、データの実体もメモリ上のオブジェクト管理区域に複製されて、新しく作成されたオブジェクトは別のメモリ上のオブジェクト管理区域のアドレスに保管される。
list_3 = copy.copy(list_1)
print(list_3)  # [-1, 1, 2, 10]
list_3[1] = 200
print(list_3)  # [-1, 200, 2, 10]
print(list_1)  # [-1, 1, 2, 10]


# 仮引数で受け取ったリストを書き換えるのではなく、受け取ったリストを完全にコピーして新しいリストを作り、それを返す関数（実引数で指定したリストには関数内で行った変更は及ばない）
def add_end(arg_list):
    import copy

    ret_list = copy.copy(arg_list)
    ret_list.append("end")
    # 変数ret_listは、ローカルスコープなので、関数の呼び出しが終わると参照できなくなる
    return ret_list


my_list = [0, 1, 2, 3]
new_list = add_end(my_list)
print(my_list)  # [0, 1, 2, 3]
print(new_list)  # [0, 1, 2, 3,'end']
