#!/usr/bin/env python # シェバン行

# 7・9・13行目は、WindowsでのUTF-8対応のために記述（Windowsシステムのデフォルト文字コードがUTF-8ではなく、UTF-8を利用しているCGIプログラム内で日本語を含む場合に文字化けが発生してしまうため）。CGIプログラムは、標準出力という仕組みでWebブラウザへのレスポンスを作り、7・9・13行目は、標準出力で利用する文字コードをUTF-8に変更する。
import cgi
import cgitb
import datetime
import io
import sys

# cgitbモジュールを使ってプログラムエラーをWebブラウザに表示できる様にする
cgitb.enable()
sys.stdout = io.TextIOWrapper(sys.stdout.buffer, encoding="utf-8")

# HTTPレスポンスのボディとなるHTML文書
html_body = """<!DOCTYPE html>
<html lang="ja">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>今日の運勢</title>
</head>
<body>
    <form method="get" name="month">
        <input type="number" min="1" max="12" style="width:2.5rem" name="month">月
        <input type="number" min="1" max="31" style="width:2.5rem" name="day">日
        &nbsp;<input type="submit" value="送信">
    </form>
    <p>{month}月{day}日生まれのあなたの今日の運勢は、{fortune}です。</P>
</body>
</html>"""

# cgiモジュールのFieldStorageクラスをインスタンス化すると、URLから、?に続く文字列であるクエリパラメータ（month=値&day=値）を取得できる（GETを使ったHTTPリクエスト）。文字列型なので整数型に変換。
param_data = cgi.FieldStorage()  # クエリパラメータ
month = int(param_data.getvalue("month"))  # クエリパラメータ名monthの値
day = int(param_data.getvalue("day"))  # クエリパラメータ名dayの値
# datetimeモジュールを利用して、現在の日時を取得
today = datetime.date.today()
# print(today.day)

# today_fortune = random.choice(["大吉", "中吉", "吉", "末吉", "凶", "大凶"])
contents = {}
contents["month"] = month
contents["day"] = day
# 辞書型の変数「contents」のキー「fortune」の値の内、２つ目のリストに０～５の数字が入り、それが１つ目のリストのインデックスを指定して、そのインデックスに対応した１つ目のリストの要素がキー「fortune」の値になる。
contents["fortune"] = ["大吉", "中吉", "吉", "末吉", "凶", "大凶"][
    today.day * month * day % 6
]
"""
contents = {
    "month": month,
    "day": day,
    "fortune": ["大吉", "中吉", "吉", "末吉", "凶", "大凶"][
        today.day * month * day % 6
    ],
}
"""

# HTTPレスポンスのヘッダ
print("Content-type: text/html")  # MIME形式表記でのコンテンツの内容
# 空白行
print("")
# HTTPレスポンスのボディ
# 関数を呼び出す時に、リスト型とタプル型には「*」、辞書型には「**」を前に付けて引数に指定すると、リスト型、タプル型、辞書型を展開してそれぞれの要素を関数の引数にまとめて渡すことができる。辞書型の場合、要素のキーが引数名、要素の値が引数の値として展開されてそれぞれ個別の引数として渡される。
print(html_body.format(**contents))

"""
value = {
    "fortune": ["大吉", "中吉", "吉", "末吉", "凶", "大凶"][2],
}
print(value)
"""
