import collections
import importlib
import itertools
import random
import sqlite3

import class_kame2

# connectメソッドでデータベースに接続すると、引数で指定したデータベースファイルが存在しない場合には、Pythonファイルと同じ階層に作成される
conn = sqlite3.connect("my_database.db")

# execute()メソッドは、引数に取った文字列をSQL文としてデータベースで実行する。executeメソッドの戻り値として、sqlite3.Cursorオブジェクトが返される。データベースファイルにすでに同じ名前のテーブルがあると、下記の様に、SQL文のcreate文で同じ名前のテーブルを作成しようとするとエラーになる
# SQL文のcreate文「create テーブル テーブル名(列名 データ型,・・・)」。realは、浮動小数点型。
# conn.execute("create table data_table(id integer,random_val real)")

# itertoolsモジュールのcount()メソッドは、引数に指定した数字から順番に数字を取得する。引数に何も指定していない場合には、0から順番に数字を取得する。
iter_cnt = itertools.count(1)
num = random.random()  # Random()メソッドだと、Random型になってしまう
# SQLインジェクション対策として、値（values）を挿入する箇所で「?」（プレースホルダ）を使用し、execute()メソッドの第2引数にプレースホルダの数と同じ要素数のタプルを指定している。プレースホルダの数とタプルの要素数が違う場合にはエラーになる。
# 組み込み関数next()関数は、イテレータ（リストなどの連続したデータ構造に対して、一つずつ順番にアクセスするための仕組み）から先頭要素を取得するために用いられる。next()関数で要素を取得した場合、その要素はイテレータの中から消滅する。
conn.execute("insert into data_table(id,random_val) values(?,?)", (next(iter_cnt), num))
for i in range(500):
    num = random.random()
    conn.execute(
        "insert into data_table(id,random_val) values(:id,:random_val)",
        {"id": next(iter_cnt), "random_val": num},
    )
# SQLインジェクション対策として、値を挿入する箇所に名前付きパラメータを指定し、execute()メソッドの第2引数に名前付きパラメータ全てのキーを持つ辞書型を指定する方法もある
"""
conn.execute(
    "insert into data_table(id,random_val) values(:id,:random_val)",
    {"id": next(iter_cnt), "random_val": num},
)
"""
# format()メソッドを使って値を挿入する箇所に指定をすると、SQLインジェクションに対して脆弱性がある
# conn.execute("insert into data_table values({},{})".format(next(iter_cnt), num))

cur = conn.execute("select count(*) from data_table")
for row in cur:
    print(row)
cur = conn.execute("select * from data_table where id=501")
for row in cur:
    print(row)
cur = conn.execute("update data_table set id=-99 where id=501")
cur = conn.execute("select * from data_table where id=-99")
for row in cur:
    print(row)
cur = conn.execute("delete from data_table where id=-99")
cur = conn.execute("select count(*) from data_table")
for row in cur:
    print(row)
# cur = conn.execute("delete from data_table")

# データベースのデータの分布を調べる
data = []
cur = conn.execute("select random_val from data_table")
for row in cur:
    data.append(int(row[0] * 10))
print(data[:10])  # スライスでリストの要素の内、最初の要素の10個を表示する
# collectionsモジュールは、リストやタプル、辞書型などの基本的なデータ構造を拡張した特殊なデータ構造を提供する。そのCounterオブジェクトは、リストなどのデータの集まりを引数に取って、その中にある要素の個数を数えてくれる。各要素をキーとし、各要素の個数を値とする辞書型のサブクラスである。
hist_data = collections.Counter(data)
for i in range(10):
    print("{:0.1f}～{:0.1f}：{}個".format(0.1 * i, 0.1 * (i + 1), hist_data[i]))

# ヒストグラムを描くclass_kame2プログラムを実行して、データベースのデータに基づきヒストグラムを描画する
importlib.reload(class_kame2)  # class_kame2モジュールを再読み込み
hist_kame = class_kame2.Kame()
hist_kame.histogram(hist_data)  # hist_dataを描画
# 下のヒストグラムを描くプログラムも行う場合には、ここでdone()メソッドを実行するとエラーになる
# hist_kame.done()

# 発生させた500個のデータを5個ずつのグループに分け、その平均値を計算する
data = []
# SQL文のgroup by関数は、指定した列で行をグループ化する
# idは、1から500までの連続値なので、これをグループ化するために100で割った余りを計算する。その余りは、0から99までの100通りになり、これを500回行えば、要素数が5個ずつのグループが100個できる。
# group by関数を使うと、データがグループ化されるので、そのままではselect関数でデータを読み込みできない。そのため、SQL文の集計関数（COUNT,SUM,AVG,MAX,MIN）が必要になる。1グループの要素の値の平均値=1グループの要素の値の合計値÷1グループの要素の数(5個)
cur = conn.execute("select avg(random_val) from data_table group by id %100")
for row in cur:
    data.append(int(row[0] * 10))
hist_data = collections.Counter(data)
# print(hist_data)
hist_kame.penup()
hist_kame.home()
hist_kame.clear()
hist_kame.pendown()
# データの分布を調べる時には、500個データがあったが、ここでは100個になっているので、グラフの高さが低くなってしまうため、引数magを10にして高さを調節。
hist_kame.histogram(hist_data, 10)
hist_kame.done()

# rollback()メソッドは、commit()メソッドで変更が確定されるまでの間、データベースへの変更を取り消す事ができる
conn.rollback()
cur = conn.execute("select count(*) from data_table")
for row in cur:
    print(row)

# commit()メソッドは、データベースへの変更を確定する
conn.commit()
# close()メソッドは、データベースとの接続を解除する
conn.close()
